// Copyright (C) 2023 WYATT GROUP
// Please see the AUTHORS file for details.
//
// super program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// super program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with super program. If not, see <https://www.gnu.org/licenses/>.

import 'package:flutter/material.dart';

/// A helper class for getting theme elements.
abstract class ThemeHelper {
  /// Gets a nullable theme element from a list of styles.
  /// {@template getElement}
  /// Styles are checked in order, and the first one that passes the
  /// [valueValidator] is returned.
  /// Style elements are transformed using the [transform] function.
  ///
  /// - [styles] : A list of styles that need to be checked.
  ///
  /// - [transform] : An optional function that transforms each style element
  /// to a [T] type after it passes the [valueValidator]. *(default: returns
  /// element as is)*
  ///
  /// - [valueValidator] : An optional validation function that
  /// determines if a style element is valid. *(default: checks if element
  /// is not null)*
  ///
  /// - [combine] : A function that combines two [P] type objects to create
  /// a new object. *(default: returns the first element)*
  ///
  /// So, if you only pass a [styles] list, the first valid style element
  /// will be returned as is.
  /// If you pass a [transform] function, the first valid style element
  /// will be transformed to a [T] type.
  /// {@endtemplate}
  static T? maybeGetElement<P, T>(
    List<P?>? styles, {
    T? Function(P?)? transform,
    bool? Function(P?)? valueValidator,
    P? Function(P?, P?)? combine,
  }) {
    // List of valid styles
    final Iterable<P?>? validStyles = styles?.where(
      (element) => valueValidator?.call(element) ?? (element != null),
    );

    // tranformation function
    final transformation = transform ?? (element) => element as T?;

    return (validStyles?.isNotEmpty ?? false)
        ? transformation.call(
            validStyles?.reduce(
              (value, element) => combine?.call(value, element) ?? value,
            ),
          )
        : null;
  }

  /// Gets a theme element from a list of styles. Throws an exception if no
  /// valid style is found.
  ///
  /// See [maybeGetElement] for more details.
  ///
  /// {@macro getElement}
  static T getElement<P, T>(
    List<P?>? styles, {
    T? Function(P?)? transform,
    bool? Function(P?)? valueValidator,
    P? Function(P?, P?)? combine,
  }) {
    final result = maybeGetElement<P, T>(
      styles,
      transform: transform,
      valueValidator: valueValidator,
      combine: combine,
    );

    if (result == null) {
      throw FlutterError(
        'No valid style found.\nPlease check your theme configuration.\n'
        'Searching for: $P in $styles (transform to $T)\n'
        'If this value can be null, use maybeGetElement instead.',
      );
    }

    return result;
  }
}
