// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart';
import 'package:wyatt_ui_components/wyatt_ui_components.dart';
import 'package:wyatt_ui_layout/src/core/scaffold_fields_wrapper.dart';
import 'package:wyatt_ui_layout/src/presentation/layouts/layout.dart';

/// {@template frame_layout}
/// A layout that contains a top app bar, a body and a bottom navigation bar.
///
/// This layout consists of a [TopAppBarComponent] at the top of the screen,
/// a [body] in the middle and a [BottomNavigationBarComponent] at the bottom.
/// You can customize the app bar and the bottom navigation bar by passing
/// a [customAppBar] and a [customBottomNavBar] functions that take
/// the corresponding components and return the customized ones.
/// {@endtemplate}
class FrameLayout extends StructuralLayout {
  /// {@macro frame_layout}
  const FrameLayout({
    required this.body,
    this.customAppBar,
    this.customBottomNavBar,
    this.customFloatingActionButton,
    this.height = 60,
    this.scaffoldFieldsWrapper,
    super.key,
  });

  /// An optional function that can be used to customize the top app bar
  final TopAppBarComponent? Function(TopAppBarComponent?)? customAppBar;

  /// An optional function that can be used to customize the bottom navigation
  final BottomNavigationBarComponent? Function(BottomNavigationBarComponent?)?
      customBottomNavBar;

  /// An optional function that can be used to customize the
  /// floating action button
  final FloatingActionButtonComponent? Function(FloatingActionButtonComponent?)?
      customFloatingActionButton;

  /// The main content of the layout.
  final Widget body;

  /// The height of the top app bar.
  final double height;

  final ScaffoldFieldsWrapper? scaffoldFieldsWrapper;

  @override
  Widget build(BuildContext context) => Scaffold(
        appBar: (customAppBar?.call(context.components.topAppBar) != null ||
                context.components.topAppBar != null)
            ? PreferredSize(
                preferredSize: Size.fromHeight(height),
                child: customAppBar?.call(context.components.topAppBar) ??
                    context.components.topAppBar!,
              )
            : null,
        floatingActionButton: customFloatingActionButton
                ?.call(context.components.floatingActionButton) ??
            context.components.floatingActionButton,
        bottomNavigationBar:
            customBottomNavBar?.call(context.components.bottomNavigationBar) ??
                context.components.bottomNavigationBar,
        body: body,
        floatingActionButtonLocation:
            scaffoldFieldsWrapper?.floatingActionButtonLocation,
        floatingActionButtonAnimator:
            scaffoldFieldsWrapper?.floatingActionButtonAnimator,
        persistentFooterButtons: scaffoldFieldsWrapper?.persistentFooterButtons,
        persistentFooterAlignment:
            scaffoldFieldsWrapper?.persistentFooterAlignment ??
                AlignmentDirectional.centerEnd,
        drawer: scaffoldFieldsWrapper?.drawer,
        onDrawerChanged: scaffoldFieldsWrapper?.onDrawerChanged,
        endDrawer: scaffoldFieldsWrapper?.endDrawer,
        onEndDrawerChanged: scaffoldFieldsWrapper?.onEndDrawerChanged,
        bottomSheet: scaffoldFieldsWrapper?.bottomSheet,
        backgroundColor: scaffoldFieldsWrapper?.backgroundColor,
        resizeToAvoidBottomInset:
            scaffoldFieldsWrapper?.resizeToAvoidBottomInset,
        primary: scaffoldFieldsWrapper?.primary ?? true,
        drawerDragStartBehavior:
            scaffoldFieldsWrapper?.drawerDragStartBehavior ??
                DragStartBehavior.start,
        extendBody: scaffoldFieldsWrapper?.extendBody ?? false,
        extendBodyBehindAppBar:
            scaffoldFieldsWrapper?.extendBodyBehindAppBar ?? false,
        drawerScrimColor: scaffoldFieldsWrapper?.drawerScrimColor,
        drawerEdgeDragWidth: scaffoldFieldsWrapper?.drawerEdgeDragWidth,
        drawerEnableOpenDragGesture:
            scaffoldFieldsWrapper?.drawerEnableOpenDragGesture ?? true,
        endDrawerEnableOpenDragGesture:
            scaffoldFieldsWrapper?.endDrawerEnableOpenDragGesture ?? true,
        restorationId: scaffoldFieldsWrapper?.restorationId,
      );
}
